package samples;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import com.obs.services.ObsClient;
import com.obs.services.ObsConfiguration;
import com.obs.services.exception.ObsException;
import com.obs.services.internal.ServiceException;
import com.obs.services.internal.utils.ConvertUtil;
import com.obs.services.internal.utils.ServiceUtils;
import com.obs.services.model.BucketCorsRule;
import com.obs.services.model.HttpMethodEnum;
import com.obs.services.model.S3BucketCors;
import com.obs.services.model.SpecialParamEnum;
import com.obs.services.model.V4TemporarySignatureRequest;
import com.obs.services.model.V4TemporarySignatureResponse;

/**
 * This sample demonstrates how to do common operations in v4 temporary signature way
 * on OBS using the OBS SDK for Java.
 */
public class V4TemporarySignatureSample
{
    private static final String endPoint = "yourdomainname";
    
    private static final String ak = "*** Provide your Access Key ***";
    
    private static final String sk = "*** Provide your Secret Key ***";
    
    private static ObsClient obsClient;
    
    private static String bucketName = "my-obs-bucket-demo";
    
    private static String objectKey = "my-obs-object-key-demo";
    
    private static CloseableHttpClient httpClient = HttpClients.createDefault();
    
    public static void main(String[] args)
    {
        ObsConfiguration config = new ObsConfiguration();
        config.setEndPoint(endPoint);
        config.setHttpsOnly(false);
        try
        {
            /*
             * Constructs a obs client instance with your account for accessing OBS
             */
            obsClient = new ObsClient(ak, sk, config);
            
            /*
             * Create bucket
             */
            doCreateBucket();
            
            /*
             * Set/Get/Delete bucket cors
             */
            doBucketCorsOperations();
            
            /*
             * Create object
             */
            doCreateObject();
            
            /*
             * Get object
             */
            doGetObject();
            
            /*
             * Set/Get object acl
             */
            doObjectAclOperations();
            
            /*
             * Delete object
             */
            doDeleteObject();
            
            /*
             * Delete bucket
             */
            doDeleteBucket();
            
        }
        catch (Exception ex)
        {
            if (ex instanceof ObsException)
            {
                ObsException e = (ObsException)ex;
                System.out.println("Response Code: " + e.getResponseCode());
                System.out.println("Error Message: " + e.getErrorMessage());
                System.out.println("Error Code:       " + e.getErrorCode());
                System.out.println("Request ID:      " + e.getErrorRequestId());
                System.out.println("Host ID:           " + e.getErrorHostId());
            }
            else
            {
                ex.printStackTrace();
            }
        }
        finally
        {
            if (httpClient != null)
            {
                try
                {
                    httpClient.close();
                }
                catch (IOException e)
                {
                }
            }
            if (obsClient != null)
            {
                try
                {
                    /*
                     * Close obs client 
                     */
                    obsClient.close();
                }
                catch (IOException e)
                {
                }
            }
        }
        
    }
    
    private static void doDeleteBucket()
        throws ObsException, ClientProtocolException, IOException
    {
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.DELETE, 300);
        v4Request.setBucketName(bucketName);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        System.out.println("Deleting bucket using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpDelete request = new HttpDelete(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request);
    }
    
    private static void doDeleteObject()
        throws ObsException, ClientProtocolException, IOException
    {
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.DELETE, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setObjectKey(objectKey);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        System.out.println("Deleting object using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpDelete request = new HttpDelete(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request);
    }
    
    private static void doObjectAclOperations()
        throws ClientProtocolException, IOException, ObsException
    {
        Map<String, Object> headers = new HashMap<String, Object>();
        headers.put("x-amz-acl", "public-read");
        
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setObjectKey(objectKey);
        v4Request.setHeaders(headers);
        v4Request.setSpecialParam(SpecialParamEnum.ACL);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        
        System.out.println("Setting object ACL to public-read using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpPut request = new HttpPut(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request);
        
        v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setObjectKey(objectKey);
        v4Request.setSpecialParam(SpecialParamEnum.ACL);
        v4Response = obsClient.createV4TemporarySignature(v4Request);
        
        HttpGet request2 = new HttpGet(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request2.addHeader(entry.getKey(), entry.getValue().toString());
        }
        System.out.println("Getting object ACL using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        
        getResponse(request2);
    }
    
    private static void doGetObject()
        throws ClientProtocolException, IOException, ObsException
    {
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setObjectKey(objectKey);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        System.out.println("Getting object using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpGet request = new HttpGet(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request);
    }
    
    private static void doCreateObject()
        throws ClientProtocolException, IOException, ObsException
    {
        Map<String, Object> headers = new HashMap<String, Object>();
        headers.put("Content-Type", "text/plain");
        
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setObjectKey(objectKey);
        v4Request.setHeaders(headers);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        
        System.out.println("Createing object using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpPut request = new HttpPut(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        request.setEntity(new StringEntity("Hello OBS", "UTF-8"));
        
        getResponse(request);
    }
    
    private static void doBucketCorsOperations()
        throws ClientProtocolException, IOException, ServiceException, ObsException
    {
        S3BucketCors s3BucketCors = new S3BucketCors();
        BucketCorsRule rule = new BucketCorsRule();
        rule.getAllowedHeader().add("Authorization");
        rule.getAllowedOrigin().add("http://www.a.com");
        rule.getAllowedOrigin().add("http://www.b.com");
        rule.getExposeHeader().add("x-obs-test1");
        rule.getExposeHeader().add("x-obs-test2");
        rule.setMaxAgeSecond(100);
        rule.getAllowedMethod().add("HEAD");
        rule.getAllowedMethod().add("GET");
        rule.getAllowedMethod().add("PUT");
        s3BucketCors.getRules().add(rule);
        
        String requestXml = ConvertUtil.transS3BucketCors(s3BucketCors);
        String requestXmlMd5 = ServiceUtils.computeMD5(requestXml);
        
        Map<String, Object> headers = new HashMap<String, Object>();
        
        headers.put("Content-Type", "application/xml");
        headers.put("Content-MD5", requestXmlMd5);
        
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setHeaders(headers);
        v4Request.setSpecialParam(SpecialParamEnum.CORS);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        
        System.out.println("Setting bucket CORS using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        
        HttpPut request = new HttpPut(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        request.setEntity(new StringEntity(requestXml, "UTF-8"));
        getResponse(request);
        
        v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
        v4Request.setBucketName(bucketName);
        v4Request.setSpecialParam(SpecialParamEnum.CORS);
        v4Response = obsClient.createV4TemporarySignature(v4Request);
        
        System.out.println("Getting bucket CORS using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        HttpGet request2 = new HttpGet(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request2.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request2);
    }
    
    private static void doCreateBucket()
        throws ObsException, ClientProtocolException, IOException
    {
        
        V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
        v4Request.setBucketName(bucketName);
        V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
        System.out.println("Creating bucket using v4 temporary signature url:");
        System.out.println("\t" + v4Response.getSignedUrl());
        
        HttpPut request = new HttpPut(v4Response.getSignedUrl());
        for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
        {
            request.addHeader(entry.getKey(), entry.getValue().toString());
        }
        getResponse(request);
    }
    
    private static void getResponse(HttpUriRequest request)
        throws ClientProtocolException, IOException
    {
        CloseableHttpResponse response = httpClient.execute(request);
        System.out.println("\tStatus:" + response.getStatusLine());
        if (response.getEntity() != null)
        {
            String content = EntityUtils.toString(response.getEntity());
            if (content == null || content.trim().equals(""))
            {
                System.out.println();
            }
            else
            {
                System.out.println("\tContent:" + content + "\n");
            }
        }
        else
        {
            System.out.println();
        }
        response.close();
    }
}
